import sys
import os
import inspect

_ARTIFACTS = []
_MODELNAMES = ['EGM96', 'GRGM360', 'MRO120F']

# COMPILER SETUP
if sys.platform == "linux" or sys.platform == "linux2":
    try:
        os.system('module load gcc') 
        # If we're on a compute cluster, intel compilers might be loaded
    except:
        print("Module load errored, compilation may fail")
        pass


os.environ['GRAVITAS_ROOT'] = os.path.dirname(
    os.path.abspath(inspect.getsourcefile(lambda: 0))
)
sys.path.insert(0, os.environ['GRAVITAS_ROOT'])
os.environ['GRAVITAS_SO'] = os.path.join(os.environ['GRAVITAS_ROOT'], 'grav.so')
_ARTIFACTS.append(os.path.join(os.environ['GRAVITAS_ROOT'], 'grav.o'))

if os.path.exists(os.environ['GRAVITAS_SO']) and 'GRAVITAS_FORCE_COMPILE' in os.environ:
    os.remove(os.environ['GRAVITAS_SO'])

if not os.path.exists(os.environ['GRAVITAS_SO']):
    cargs = '-fPIC -Wall -O3 -std=c99'
    nwithext = lambda ext: " ".join([n+f".{ext}" for n in _MODELNAMES])
    all_to_obj = [f'gcc -c {cargs} {n}.c -o {n}.o' for n in _MODELNAMES]
    _ARTIFACTS.extend([os.path.join(os.environ['GRAVITAS_ROOT'], f'{x}.o') for x in _MODELNAMES])
    cmds = [f'gcc -c {cargs} gravlib.c -o grav.o',
            *all_to_obj,
            f'gcc grav.o {nwithext("o")} -shared -Wall -o grav.so']
    os.chdir(os.environ['GRAVITAS_ROOT'])
    for cmd in cmds:
        print(cmd)
        os.system(cmd)
    for art in _ARTIFACTS:
        os.remove(art)

from .lib import *


from ctypes import *
import numpy as np
import os

lib = CDLL(os.environ['GRAVITAS_SO'])
_GRAV_FUN = lib.egm96_gravity
_SUPPORTED_MODELS = {'EGM96': 360, 'GRGM360': 360, "MRO120F": 120}
_MAX_PTS = int(1e5)

def acceleration(position_ecef: np.ndarray, max_order: int, use_model: str = "EGM96") -> np.ndarray:
    if use_model not in _SUPPORTED_MODELS:
        raise NotImplementedError(f"Model {use_model} is not supported, it must be in: {_SUPPORTED_MODELS}")
    if _SUPPORTED_MODELS[use_model] < max_order:
        raise ValueError(f"The {use_model} model has coefficients to a maximum order of {_SUPPORTED_MODELS[use_model]}, {max_order} input")
    if position_ecef.shape[0] > _MAX_PTS:
        raise ValueError(f"Currently, gravitas is limited to 1e5 points per acceleration() call ({position_ecef.shape[0]} provided)")
    xl, yl, zl = position_ecef[:,0].tolist(), position_ecef[:,1].tolist(), position_ecef[:,2].tolist()
    n = len(xl)
    _GRAV_FUN.argtypes = [POINTER(c_double * n), 
                        POINTER(c_double * n), 
                        POINTER(c_double * n), 
                        c_int,
                        c_int,
                        c_char_p]
    _GRAV_FUN.restype = POINTER(c_double * 3*n)
    x, y, z = (c_double * n)(*xl), (c_double * n)(*yl), (c_double * n)(*zl)
    res = np.array(_GRAV_FUN(x, y, z, n, max_order, use_model.encode('utf-8')).contents[:]).reshape((-1,3))
    os.environ['GRAVITAS_ROOT_CP'] = os.environ['GRAVITAS_ROOT']
    return res

